/*
 Copyright The MITRE Corporation 2009-2010.   All rights reserved.
 */

package org.mitre.jcarafe.dparser
import org.mitre.jcarafe.crf.{NonFactoredFeatureManager,FeatureReturn,ObsSource,SourceSequence,BuiltFeature,WordProperties}

abstract class DepParserFeatureManager(iString: String, var maxNumStates: Int) extends NonFactoredFeatureManager[String](iString) {
  import org.mitre.jcarafe.crf.IncrementalMurmurHash._
  
  
  

  val posPair_C     = hash("posPair:")
  val wdPair_C      = hash("wdPair:")
  val selfW_C       = hash("SELFW:")
  val selfP_C       = hash("SELFP:")
  val pPos_C        = hash("pPos:")
  val pWord_C       = hash("pWord:")
  val cPos_C        = hash("cPos:")
  val cWord_C       = hash("cWord:")
  val surround1_C     = hash("surround1:")
  val surround2_C     = hash("surround2:")
  val surround3_C     = hash("surround3:")
  val surround4_C     = hash("surround4:")
  val endOfSeq_C      = hash("END:")
  val startOfSeq_C    = hash("START:")
  val inBetweenLeft   = hash("inBetweenLeft:")
  val inBetweenRight   = hash("inBetweenRight:")
  val dist_C           = hash("dist:")
  val cluster_C        = hash("cluster:")
  val p_cluster_C      = hash("parentCluster:")
  val pWordAndPos_C = mix(pWord_C,pPos_C)
  val cWordAndPos_C = mix(cWord_C,cPos_C)
  val pAndCPos_C    = mix(pPos_C,cPos_C)
  val pAndCWord_C   = mix(pWord_C,cWord_C)
  val pAndCCluster_C = mix(cluster_C,p_cluster_C)
  val pPosCWordCPos_C   = mix(mix(pPos_C,cWord_C),cPos_C)
  val pWordPPosCWord_C  = mix(pWordAndPos_C,cWord_C)
  val pWordPPosCPos_C   = mix(pWordAndPos_C,cPos_C)
  val pWordCWordCPos_C  = mix(pAndCWord_C,cPos_C)

  def basicFns(ss: Int, src: SourceSequence[String], cp: Int, state: Int) : FeatureReturn = {
    val srcCp = src(cp).code
    val curStr = src(cp).code
    val curPos = src(cp).posCode
    val ff = if (state == cp) (BuiltFeature(selfP_C) @@ curPos) else (BuiltFeature(posPair_C) @@ src(state).posCode @@ curPos)
    val gg = if (state == cp) {
      BuiltFeature(selfW_C) @@ curStr 
    } else {
      BuiltFeature(wdPair_C) @@ src(state).obs.toString @@ curStr
    }
    new FeatureReturn(ff) join gg
  }

  def basicFns2(ss: Int, src: SourceSequence[String], cp: Int, state: Int) : FeatureReturn = {
    if (cp > 0) {
      val prevPos1 = src(cp-1).posCode
      val prev1Tok = src(cp-1).obs.toString
      new FeatureReturn(BuiltFeature(pPos_C) @@ prevPos1) join (BuiltFeature(pWord_C) @@ prev1Tok)
    }
    else new FeatureReturn
  }

  private def getDistance(cp: Int, par: Int) = {
    val ad = math.abs(cp - par)
    if (ad < 6) cp - par
    else if (cp < par) 6
    else -6
  }
  
  private def getClusterCode(s: String) = {
    wdProps match {
      case Some(wp) => wp.get(s) match {case Some(a :: _) => hash(a) case _ => 0L}
      case None => 0L
    }
  }
  
  def unigramFns(ss: Int, src: SourceSequence[String], cp: Int, state: Int) : FeatureReturn = {
   val curStr = src(cp).code
   val curPos = src(cp).posCode
   val pStr = src(state).code
   val pPos = src(state).posCode
   val dd = getDistance(cp,state)
   val cpCl = getClusterCode(src(cp).obs)  
   val stateCl = getClusterCode(src(state).obs)

   (new FeatureReturn(BuiltFeature(pPos_C) @@ pPos)) 
     .join (BuiltFeature(pWord_C) @@ pStr) 
     .join (BuiltFeature(cPos_C) @@ curPos) 
     .join (BuiltFeature(cWord_C) @@ curStr) 
     .join (BuiltFeature(pWordAndPos_C) @@ pStr @@ pPos) 
     .join (BuiltFeature(cWordAndPos_C) @@ curStr @@ curPos)
     .join (BuiltFeature(pWordAndPos_C) @@ pStr @@ pPos @@ cluster_C @@ stateCl)
     .join (BuiltFeature(pPos_C) @@ pPos @@ dist_C @@ dd)
     .join (BuiltFeature(pWord_C) @@ pStr @@ dist_C @@ dd) 
     .join (BuiltFeature(cPos_C) @@ curPos @@ dist_C @@ dd) 
     .join (BuiltFeature(cWord_C) @@ curStr @@ dist_C @@ dd) 
     .join (BuiltFeature(pWordAndPos_C) @@ pStr @@ pPos @@ dist_C @@ dd) 
     .join (BuiltFeature(cWordAndPos_C) @@ curStr @@ curPos @@ dist_C @@ dd)
     .join (BuiltFeature(pWordAndPos_C) @@ pStr @@ pPos @@ cluster_C @@ stateCl @@ dist_C @@ dd)

  }

  def bigramFns(ss: Int, src: SourceSequence[String], cp: Int, state: Int) : FeatureReturn = {
   val curStr = src(cp).code
   val curPos = src(cp).posCode
   val pStr = src(state).code
   val pPos = src(state).posCode
   val dd = getDistance(cp,state)
   val cpCl = getClusterCode(src(cp).obs)
   val stateCl = getClusterCode(src(state).obs)
    new FeatureReturn(BuiltFeature(pAndCPos_C) @@ pPos @@ curPos)
      .join (BuiltFeature(pAndCWord_C) @@ pStr @@ curStr)
      .join (BuiltFeature(pAndCCluster_C) @@ cpCl @@ stateCl)
      .join (BuiltFeature(pWordPPosCWord_C) @@ pStr @@ pPos @@ curStr) 
      .join (BuiltFeature(pWordPPosCPos_C) @@ pStr @@ pPos @@ curPos) 
      .join (BuiltFeature(pWordCWordCPos_C) @@ pStr @@ curStr @@ curPos) 
      .join (BuiltFeature(pPosCWordCPos_C) @@ pPos @@ curStr @@ curPos)
      .join (BuiltFeature(pPosCWordCPos_C) @@ pPos @@ curStr @@ curPos @@ cpCl @@ stateCl)
      .join (BuiltFeature(pAndCPos_C) @@ pPos @@ curPos @@ dist_C @@ dd) 
      .join (BuiltFeature(pAndCWord_C) @@ pStr @@ curStr @@ dist_C @@ dd) 
      .join (BuiltFeature(pWordPPosCWord_C) @@ pStr @@ pPos @@ curStr @@ dist_C @@ dd) 
      .join (BuiltFeature(pWordPPosCPos_C) @@ pStr @@ pPos @@ curPos @@ dist_C @@ dd) 
      .join (BuiltFeature(pWordCWordCPos_C) @@ pStr @@ curStr @@ curPos @@ dist_C @@ dd) 
      .join (BuiltFeature(pPosCWordCPos_C) @@ pPos @@ curStr @@ curPos @@ dist_C @@ dd)
  }

  def distDirFns(ss:Int, src: SourceSequence[String], cp: Int, state: Int) : FeatureReturn = {
    val dval = (cp - state)
    val dir = if (dval > 0) 1 else if (dval == 0) 0 else -1
    val abval = dval.abs
    val curStr = src(cp).code
    val df = BuiltFeature(dir) @@ curStr
    val cl = getClusterCode(src(cp).obs)
    new FeatureReturn(df) join (df @@ abval) join (BuiltFeature(dir) @@ cl @@ abval) join (BuiltFeature(curStr) @@ abval)
  }

  def prefixFns(ss:Int, src: SourceSequence[String], cp: Int, state: Int) : FeatureReturn = {
    val curStr = src(cp).obs.toString
    new FeatureReturn
  }

  def inBetweenFns(ss: Int, src: SourceSequence[String], cp: Int, state: Int) : FeatureReturn = {
    val srcCp = src(cp)
    val curStr = src(cp).code
    val curPos = src(cp).posCode
    val pStr = src(state).code
    val pPos = src(state).posCode
    val dir = if (state < cp) -1 else 1
    val st = math.min(state,cp)
    val en = math.max(state,cp)
    val initF = if (dir < 0) BuiltFeature(inBetweenLeft) @@ curPos @@ pPos else BuiltFeature(inBetweenRight) @@ curPos @@ pPos
    var fr = new FeatureReturn
    val dd = getDistance(cp,state)
    var i = st + 1
      while (i < en) {
        fr = fr join (initF @@ src(i).posCode) join (initF @@ src(i).posCode @@ dist_C @@ dd)
        i += 1
      }
    fr
  }
  
  def surroundingWordPosFns(ss: Int, src: SourceSequence[String], cp: Int, state: Int) : FeatureReturn = {
    val ln = src.length
    val cPos = src(cp).posCode
    val cPosPlus1 = if (cp < ln - 1) src(cp+1).posCode else endOfSeq_C
    val cPosMinus1 = if (cp > 0) src(cp-1).posCode else startOfSeq_C
    val pPos = src(state).posCode
    val pPosPlus1 = if (state < ln - 1) src(state+1).posCode else endOfSeq_C
    val pPosMinus1 = if (state > 0) src(state-1).posCode else startOfSeq_C
    val dd = getDistance(cp,state)
    new FeatureReturn(BuiltFeature(surround1_C) @@ pPos @@ pPosPlus1 @@ cPosMinus1 @@ cPos)
      .join (BuiltFeature(surround2_C) @@ pPosMinus1 @@ pPos @@ cPosMinus1 @@ cPos)
      .join (BuiltFeature(surround3_C) @@ pPos @@ pPosPlus1 @@ cPos @@ cPosPlus1)
      .join (BuiltFeature(surround4_C) @@ pPosMinus1 @@ pPos @@ cPos @@ cPosPlus1)
      .join (BuiltFeature(surround1_C) @@ pPos @@ pPosPlus1 @@ cPosMinus1 @@ cPos @@ dist_C @@ dd)
      .join (BuiltFeature(surround2_C) @@ pPosMinus1 @@ pPos @@ cPosMinus1 @@ cPos @@ dist_C @@ dd)
      .join (BuiltFeature(surround3_C) @@ pPos @@ pPosPlus1 @@ cPos @@ cPosPlus1 @@ dist_C @@ dd)
      .join (BuiltFeature(surround4_C) @@ pPosMinus1 @@ pPos @@ cPos @@ cPosPlus1 @@ dist_C @@ dd)
  }
  // these need to be added "by hand" in the code and can not be specified through a config file
  // change this in the future
  val fnList = Nil  

}
