package org.mitre.jcarafe.dparser

import org.mitre.jcarafe.crf.{Trainer,InstanceSequence,CondLogLikelihoodLearner,NonFactoredSerializer,
  GenericNonFactoredTrainer, NonFactoredFeatureRep, NonFactoredTrainingSeqGen, NonFactoredDecoder,
  DecodingNonFactoredFeatureRep,NonFactoredDecodingSeqGen, PsaLearner, SeqGenScorer, WordProperties
}
import org.mitre.jcarafe.tagger.TaggerTask

trait ProjectiveMstCrfTraining[O] extends Trainer[O] {
  
  def trainingRoutine(seqs: Seq[InstanceSequence]) = {
    println("Processed " + seqs.length + " sentences . . . beginning parameter estimation..\n")
    println("Number of features = " + sGen.getNumberOfFeatures)
    val mstCrf =
      if (opts.psa)
        new StochasticProjectiveMstCrf(sGen.getNumberOfFeatures, opts) with PsaLearner
      else
    	new ProjectiveMstCrf(sGen.getNumberOfFeatures, opts.gaussian) with CondLogLikelihoodLearner
    trainModel(mstCrf,seqs)
  }
}

class ProjectiveDependencyParser(argv: Array[String]) extends TaggerTask[String](argv) {

  import NonFactoredSerializer._

  lazy val trainer = new GenericNonFactoredTrainer[String](true,opts) with ProjectiveMstCrfTraining[String] {
    // should get rid of this for dependency parsing - number of states varies by sentence 
    // this just acts as a global upperbound at the moment - so pick a high value like 100
    val nstates = opts.numStates.getOrElse(100) 
    val mgr = new DynamicDepParserFeatureManager(opts.nonFactoredFeatureSpec.get,nstates)
    mgr.wdProps match {
      case None => opts.wordPropFile match {case Some(f) => mgr.wdProps_=(Some(new WordProperties(f))) case None => } case Some(_) => }
    val fr = new NonFactoredFeatureRep[String](mgr,false,opts.numStates.getOrElse(0),true)
    //val sGen = new NonFactoredTrainingSeqGen[String] (fr, opts) with XmlParserSeqGen
    val sGen = new NonFactoredTrainingSeqGen[String] (fr, opts) with CoNLLSeqGen
  }
  
  lazy val decoder = new NonFactoredDecoder[String](true,opts) {
    val model = readModel(opts.model.get)
    model.fixAlphabet(true)
    println("loaded in model: " + opts.model.get)
    opts.modelDump match {case Some(mf) => model.print(new java.io.File(mf)) case None => }
    override lazy val viterbiDecoder = new ProjectiveMstInference(model.crf)
    val mgr = new DynamicDepParserFeatureManager(model.fspec,model.numStates)
    mgr.wdProps match {
      case None => mgr.wdProps_=(model.wp) case Some(_) =>
    }
    val fr = new DecodingNonFactoredFeatureRep[String](model.faMap,mgr,model.numStates,true)
    //val sGen = new NonFactoredDecodingSeqGen[String] (fr, model, opts) with XmlParserSeqGen
    val sGen = new NonFactoredDecodingSeqGen[String] (fr, model, opts) with CoNLLSeqGen
  }	    
}

object ProjectiveParserTask {
  
  def main(argv: Array[String]) : Unit = {
    val tagger = new ProjectiveDependencyParser(argv) 
    tagger.process()
  }
  
}